package Question3;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;


/**
 * This class maintains a list of songs (representing a song playlist) that can
 * be added to and sorted by various fields.
 */
public class SongPlaylist extends Playlist<Object>{
	
	private List<Song> songList; // List of songs in this playlist
	
	 // Default constructor. Initializes an empty playlist.
	public SongPlaylist() {
		songList = new ArrayList<Song>();
	}
	
	
	 // Sort the songs in this playlist in alphabetical order by song title.
	public void sortByTitle() {
		Collections.sort(songList);
	}
	
	@Override
	public void addToPlaylist(Object playlistObject) {
		// TODO Auto-generated method stub
		songList.add((Song) playlistObject);
	}
	
	
	// Sort the songs in this playlist in alphabetical order by artist.
	// Comparator Class
    static final Comparator<Song> ARTIST_ORDER = new Comparator<Song>() {
        public int compare(Song s1, Song s2) {
            return s1.getArtist().compareTo(s2.getArtist());
        }
    };

    // Sorting by artist class
	public void sortByArtist() {
		Collections.sort(songList, ARTIST_ORDER);
	}
	
	
	
	
	
	// Sort the songs in this playlist in ascending order by year.
	// Comparator Class
    static final Comparator<Song> YEAR_ORDER = new Comparator<Song>() {
        public int compare(Song s1, Song s2) {
            return s1.getYear() - s2.getYear();
        }
    };
    
    // Sorting by year method
	public void sortByYear() {
		Collections.sort(songList, YEAR_ORDER);
	}
	
	
	
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer(String.format(
				"%-18s | %-22s | %-6s%n", "ARTIST", "SONG TITLE", "YEAR"));
		sb.append("-------------------+------------------------+-------\n");
		for (Song song : songList) {
			sb.append(String.format("%-18s | %-22s | %-15s%n",
					song.getArtist(), song.getTitle(), song.getYear()));
		}
		return sb.toString();
	}
	
	/**
	 * Main method. Use this to test your code.
	 * @param args Not used
	 */
	public static void main(String[] args) {
		
		// Populate the playlist with test songs
		SongPlaylist playlist = new SongPlaylist();
		playlist.addToPlaylist(new Song("Paranoid Android", "Radiohead", 1997));
		playlist.addToPlaylist(new Song("Machine Gun", "Slowdive", 1993));
		playlist.addToPlaylist(new Song("A Change Is Gonna Come", "Sam Cooke", 1964));
		playlist.addToPlaylist(new Song("15 Step", "Radiohead", 2008));
		playlist.addToPlaylist(new Song("Blue Line Swinger", "Yo La Tengo", 1995));
		playlist.addToPlaylist(new Song("Reckoner", "Radiohead", 2008));
		playlist.addToPlaylist(new Song("Ashes To Ashes", "David Bowie", 1980));
		playlist.addToPlaylist(new Song("Speed Law", "Mos Def", 1999));
		playlist.addToPlaylist(new Song("93 'Til Infinity", "Souls of Mischief", 1993));
		playlist.addToPlaylist(new Song("Season of the Shark", "Yo La Tengo", 2003));
		playlist.addToPlaylist(new Song("Cet Air-La", "France Gall", 1966));
		playlist.addToPlaylist(new Song("Space Oddity", "David Bowie", 1969));
		
		// Display the unsorted playlist
		System.out.println();
		System.out.println("UNSORTED PLAYLIST:");
		System.out.println();
		System.out.println(playlist);
		
		
		// Display the playlist sorted by artist
		playlist.sortByArtist();
		System.out.println();
		System.out.println("PLAYLIST SORTED BY ARTIST:");
		System.out.println();
		System.out.println(playlist);
		
		
		// Display the playlist sorted by title
		playlist.sortByTitle();
		System.out.println();
		System.out.println("PLAYLIST SORTED BY SONG TITLE:");
		System.out.println();
		System.out.println(playlist);
		
		// Display the playlist sorted by year
		playlist.sortByYear();
		System.out.println();
		System.out.println("PLAYLIST SORTED BY YEAR:");
		System.out.println();
		System.out.println(playlist);
		
	}

}
